/*
 * API forecast.io
 * Sanovation AG
 *
 * Written by: Thomas Dobler and Eric Neher
 *
 * Date: 2014-04-20
 */

//////////////////////////////////////////
// CONSTANTS

// API Key and URL to forecast.io
var API_KEY_FORECAST = "f56a819887b49aad829f532bd2f49bc3";
var UNITS = "si"; // "si" or "us"
var URI_FIX = "https://api.forecast.io/forecast/"+API_KEY_FORECAST+"/";


// API Key and URL to Google Geocoding API
var API_KEY_GOOGLE = "AIzaSyC8B2bWHvHvMPB1bpSiPd3XYNhpcVw81fw";
var GOOGLE_API_FIX_add2latlng = "https://maps.googleapis.com/maps/api/geocode/json?key= " + API_KEY_GOOGLE + " &address=";
var GOOGLE_API_FIX_latlng2add = "https://maps.googleapis.com/maps/api/geocode/json?key= " + API_KEY_GOOGLE + " &latlng=";


//////////////////////////////////////////
// API

/**
 * Wrapper Method
 * Makes a request to forecast.io  based on a location and the current time
 * @param {Object} LATITUDE
 * @param {Object} LONGITUDE
 * @param {Object} CALLBACK_SETWEATHERDATA
 */
function getCurrentWeather( LATITUDE, LONGITUDE, CALLBACK_SETWEATHERDATA ){
    _getCurrentWeather(LATITUDE, LONGITUDE, CALLBACK_SETWEATHERDATA);
}

/**
 * Wrapper Method
 * Makes a request to forecast.io  based on a location and a given time
 * @param {Object} LATITUDE
 * @param {Object} LONGITUDE
 * @param {Object} DAY
 * @param {Object} MONTH
 * @param {Object} YEAR
 * @param {Object} HOUR
 * @param {Object} CALLBACK_SETWEATHERDATA
 */
function getWeatherByTime( LATITUDE, LONGITUDE, DAY, MONTH, YEAR, HOUR, CALLBACK_SETWEATHERDATA){
    _getWeatherByTime(LATITUDE, LONGITUDE, DAY, MONTH, YEAR, HOUR, CALLBACK_SETWEATHERDATA);
}

/**
 * Wrapper Method
 * Makes a request to forecast.io based on a location and given unix timestamp
 * @param {Object} LATITUDE
 * @param {Object} LONGITUDE
 * @param {Object} TIME
 * @param {Object} CALLBACK_SETWEATHERDATA
 */
function getWeatherByTimeDirect( LATITUDE, LONGITUDE, TIME, CALLBACK_SETWEATHERDATA){
    _getWeatherByTimeDirect(LATITUDE, LONGITUDE, TIME, CALLBACK_SETWEATHERDATA);
}


/**
 * Wrapper Method
 * Makes a request to google geocoding api based on a written address
 * @param {Object} ADDRESS
 */
function getLatitudeByAddress( ADDRESS ){
    return _getLatitudeByAddress( ADDRESS );
}


/**
 * Wrapper Method
 * Makes a request to google geocoding api based on a written address
 * @param {Object} ADDRESS
 */
function getLongitudeByAddress( ADDRESS ){
    return _getLongitudeByAddress( ADDRESS );
}

/**
 * Wrapper Method
 * Makes a request to google geocoding api based on coordinates
 * @param {Object} LAT
 * @param {Object} LNG
 */
function getAddressByLatAndLng( LAT, LNG ){
    return _getAddressByLatAndLng( LAT, LNG );
}

/**
 * Wrapper Method
 * Makes a request to google geocoding api based on a written address
 * @param {Object} ADDRESS
 */
function getAddressByAddress( ADDRESS ){
    return _getAddressByAddress( ADDRESS );
}


//////////////////////////////////////////

/**
 * Transforms to an general ajax request
 */
function _getCurrentWeather(LATITUDE, LONGITUDE, CALLBACK_SETWEATHERDATA){
    _asyncForecastRequest(""+LATITUDE+","+LONGITUDE, CALLBACK_SETWEATHERDATA);
}

/**
 * Transforms to an general ajax request
 * Handling some date problems
 */
function _getWeatherByTime(LATITUDE, LONGITUDE, DAY, MONTH, YEAR, HOUR, CALLBACK_SETWEATHERDATA){
    if (DAY < 10) DAY="0" + DAY;
    if (MONTH < 10 ) MONTH = "0" + MONTH;
    if (YEAR < 1000) YEAR = +YEAR + +2000;
    _asyncForecastRequest(""+LATITUDE+","+LONGITUDE+","+YEAR+"-"+MONTH+"-"+DAY+"T"+HOUR+":00:00", CALLBACK_SETWEATHERDATA);
}

/**
 * Transforms to an general ajax request
 */
function _getWeatherByTimeDirect(LATITUDE, LONGITUDE, TIME, CALLBACK_SETWEATHERDATA){
    _asyncForecastRequest(""+LATITUDE+","+LONGITUDE+","+TIME, CALLBACK_SETWEATHERDATA);
}

/**
 * Makes an synchronous request. Expects an json result.
 * Returns the latitude.
 * @param {Object} ADDRESS
 */
function _getLatitudeByAddress(ADDRESS){
	var result = "";
	try{
	    var xmlhttp = createRequest();
	    xmlhttp.open("GET",GOOGLE_API_FIX_add2latlng+ADDRESS+"&sensor=false",false);
	    xmlhttp.send();
	    result = JSON.parse(xmlhttp.responseText);
	    result = result.results[0].geometry.location.lat;
    }
    catch(Err){
    	
    	result = undefined;
    }
    return result;
}
/**
 * Makes an synchronous request. Expects an json result.
 * Returns the longitude.
 * @param {Object} ADDRESS
 */
function _getLongitudeByAddress(ADDRESS){
	var result = "";
	try{
	    var xmlhttp = createRequest();
	    xmlhttp.open("GET",GOOGLE_API_FIX_add2latlng+ADDRESS+"&sensor=false",false);
	    xmlhttp.send();
	    result = JSON.parse(xmlhttp.responseText);
	    result = result.results[0].geometry.location.lng;
    }
    catch(Err){
    	result = undefined;
    }
    return result;
}
/**
 * Makes an synchronous request. Expects an json result.
 * Returns a written address.
 * @param {Object} ADDRESS
 */
function _getAddressByLatAndLng(LAT, LNG ){
	var result = "";
	try{
	    var xmlhttp = createRequest();
	    xmlhttp.open("GET",GOOGLE_API_FIX_latlng2add + LAT + "," + LNG + "&sensor=false",false);
	    xmlhttp.send();
	    result = JSON.parse(xmlhttp.responseText);
	    var split = result.results[0].formatted_address.split(",");
	    if (split.length == 3){
	    	result =  split[1] + ", " + split[2];
	    }
	    else{
	    	result = result.results[0].formatted_address;
	    }
	}
    catch(Err){
    	result = undefined;
    }
    return result;
}
/**
 * Makes an synchronous request. Expects an json result.
 * Returns the written address.
 * @param {Object} ADDRESS
 */
function _getAddressByAddress( ADDRESS ){
	var result="";
	try{
		var xmlhttp = createRequest();
	    xmlhttp.open("GET",GOOGLE_API_FIX_add2latlng + ADDRESS + "&sensor=false",false);
	    xmlhttp.send();
	    result = JSON.parse(xmlhttp.responseText);
	    result = result.results[0].formatted_address;
    }
    catch(Err){
    	result = undefined;
    }
    return result;
}
//////////////////////////////////////////

   


// private function

function createRequest(){
    if (window.XMLHttpRequest)
    {// code for IE7+, Firefox, Chrome, Opera, Safari
        return xmlhttp = new XMLHttpRequest();
    }
    else
    {// code for IE6, IE5
        return xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
    }
}

/**
 * Makes an asynchronous request with Ext.data.JsonP.request()
 * Passes the json to the passed-in CALLBACK_SETWEATHERDATA method.
 * @param {Object} URL_VARI
 * @param {Object} CALLBACK_SETWEATHERDATA
 */
function _asyncForecastRequest(URL_VARI, CALLBACK_SETWEATHERDATA){
	//console.log("invoke async forecast request");
    //console.log("try to get JSON result by:"+ URI_FIX+URL_VARI+"?units="+UNITS);
    Ext.data.JsonP.request({
                    url: URI_FIX+URL_VARI+"?units="+UNITS,
                    callbackKey: 'callback',
                    success: function(result, request) {
                    	CALLBACK_SETWEATHERDATA( result );
                    },
                    failure: function(result,request){
                    	CALLBACK_SETWEATHERDATA( undefined );
                    }
                    });
 
}
/**
 * Return a readable string from a unixtime.
 * @param {Object} unixtime
 */
function unixToGMTLocal(unixtime){
    var dateCurrently = new Date(unixtime*1000);
    var day = dateCurrently.getDate();
    var month = dateCurrently.getMonth() + 1;
    var year = dateCurrently.getFullYear();
    var hours = dateCurrently.getHours();
    var minutes = dateCurrently.getMinutes();
    var seconds = dateCurrently.getSeconds();
    return day +"-"+month+"-"+year+" - "+hours + ':' + minutes + ':' + seconds;
    
}
